/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) <Year>, <Your Name Here>
 *
 * <Your Name Here> <<Your Email Here>>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief JabberAuthorize application
 *
 * \author Philippe Sultan <philippe.sultan@inria.fr>
 * 
 * A jabber interface to place jingle talks through Asterisk
 * \ingroup applications
 */

/*** MODULEINFO
	<defaultenabled>no</defaultenabled>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 40722 $")

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/app.h"
#include "asterisk/jabber.h"

static char *app = "JabberAuthorize";
static char *synopsis = 
"Jabber Number Composing Application.";
static char *descrip = "This application helps jabber users dial extensions and pass calls through Asterisk.\n";

enum {
	OPTION_A = (1 << 0),
	OPTION_B = (1 << 1),
	OPTION_C = (1 << 2),
} option_flags;

enum {
	OPTION_ARG_B = 0,
	OPTION_ARG_C = 1,
	/* This *must* be the last value in this enum! */
	OPTION_ARG_ARRAY_SIZE = 2,
} option_args;

AST_APP_OPTIONS(app_opts,{
	AST_APP_OPTION('a', OPTION_A),
	AST_APP_OPTION_ARG('b', OPTION_B, OPTION_ARG_B),
	AST_APP_OPTION_ARG('c', OPTION_C, OPTION_ARG_C),
});


static int jabber_authorize_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct ast_module_user *u;
	struct aji_client *client = NULL;
	struct aji_buddy *buddy = NULL;
	char *cid = NULL;
	char *pcid = NULL;	
	char *parse;

	AST_DECLARE_APP_ARGS(args,
		AST_APP_ARG(client);
	);

	if (ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "%s requires an argument ({jabber client})\n", app);
		return -1;
	}

	u = ast_module_user_add(chan);
	parse = ast_strdupa(data);
	AST_STANDARD_APP_ARGS(args, parse);
	if (args.argc >= 2) {
		ast_log(LOG_WARNING, "Options are not accepted\n");
		return -1;
	}

	if (ast_strlen_zero(args.client)) {
		ast_log(LOG_ERROR, "Jabber client parameter is missing\n");
		return -1;
	}

	/* cid contains the full jid. pcid contains jid without resource */
	cid = ast_strdupa(chan->cid.cid_name);
	pcid = ast_strdupa(chan->cid.cid_name);
	pcid = strsep(&pcid, "/");

	client = ast_aji_get_client(args.client);
	if (!client) {
		ast_log(LOG_WARNING, "Jabber client %s not found in jabber configuration file!\n", args.client);
		return -1;
	}
	else {
		ast_log(LOG_DEBUG, "Using client '%s' to chat with user '%s'\n", client->name, cid);
	}

	/* user authorization relies on our buddy list*/
	if(!&client->buddies) {
		ast_log(LOG_WARNING, "No buddies for connection.\n");
		return -1;
	}
	/* check if sender is allowed to chat with us*/
	buddy = ASTOBJ_CONTAINER_FIND(&client->buddies, pcid);
	if (!buddy) {
		ast_log(LOG_WARNING, "Could not find buddy %s in list.\n", pcid);
		ast_aji_send(client, cid, "You are not authorized to place calls on this system");
		return -1;
	}

	ast_aji_send(client, cid, "Enter the number you wish to call");

	ast_module_user_remove(u);

	return res;
}

static int unload_module(void)
{
	int res;
	res = ast_unregister_application(app);
	return res;	
}

static int load_module(void)
{
	return ast_register_application(app, jabber_authorize_exec, synopsis, descrip);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Jabber Number Composing Application");
